@@ -38,8 +38,16 @@ createMcuLineIndices param imgWidth mcuWidth =
         compH = fromIntegral $ verticalSamplingFactor param
         imageBlockSize = toBlockSize imgWidth
 
-        indexSolo = take (imageBlockSize * compH) [0 ..]
-        indexMulti = 
+        -- if the displayed MCU block is only displayed in half (like with
+        -- width 500 then we loose one macroblock of the MCU at the end of
+        -- the line. Previous implementation which naively used full mcu
+        -- was wrong. Only taking into account visible macroblocks
+        indexSolo = [base + x
+            | y <- [0 .. compH - 1]
+            , let base = y * mcuWidth * compW
+            , x <- [0 .. imageBlockSize - 1]]
+
+        indexMulti =
             [(mcu + y * mcuWidth) * compW + x
                 | mcu <- [0 .. mcuWidth - 1]
                 , y <- [0 .. compH - 1]
@@ -129,7 +137,7 @@ decodeRefineAc params _ block eobrun
                    performEobRun $ idx + 1
           else
             performEobRun $ idx + 1
-                   
+
         unpack idx | idx > maxIndex = pure 0
         unpack idx = do
             rrrrssss <- decodeRrrrSsss $ acHuffmanTree params
@@ -143,7 +151,7 @@ decodeRefineAc params _ block eobrun
                   let newEobRun = (1 `unsafeShiftL` r) + lowBits - 1
                   performEobRun idx
                   pure newEobRun
-                         
+
               (  r, _) -> do
                   val <- getBitVal
                   idx' <- updateCoeffs (fromIntegral r) idx
